@@ -14,21 +14,24 @@ import logging
 import functools
 
 from ibm_botocore import waiter, xform_name
+from ibm_botocore.args import ClientArgsCreator
 from ibm_botocore.auth import AUTH_TYPE_MAPS
 from ibm_botocore.awsrequest import prepare_request_dict
 from ibm_botocore.docs.docstring import ClientMethodDocstring
 from ibm_botocore.docs.docstring import PaginatorDocstring
-from ibm_botocore.exceptions import ClientError, DataNotFoundError
-from ibm_botocore.exceptions import OperationNotPageableError
-from ibm_botocore.exceptions import UnknownSignatureVersionError
+from ibm_botocore.exceptions import (
+    ClientError, DataNotFoundError, OperationNotPageableError,
+    UnknownSignatureVersionError, InvalidEndpointDiscoveryConfigurationError,
+    UnknownFIPSEndpointError,
+)
 from ibm_botocore.hooks import first_non_none_response
 from ibm_botocore.model import ServiceModel
 from ibm_botocore.paginate import Paginator
-from ibm_botocore.utils import CachedProperty
-from ibm_botocore.utils import get_service_module_name
-from ibm_botocore.utils import S3RegionRedirector
-from ibm_botocore.utils import S3ArnParamHandler
-from ibm_botocore.utils import S3EndpointSetter
+from ibm_botocore.utils import (
+    CachedProperty, get_service_module_name, S3RegionRedirector,
+    S3ArnParamHandler, S3EndpointSetter, ensure_boolean,
+    S3ControlArnParamHandler, S3ControlEndpointSetter,
+)
 from ibm_botocore.args import ClientArgsCreator
 from ibm_botocore import UNSIGNED
 # Keep this imported.  There's pre-existing code that uses
@@ -88,9 +91,13 @@ class ClientCreator(object):
         self._register_s3_events(
             service_client, endpoint_bridge, endpoint_url, client_config,
             scoped_config)
+        self._register_s3_control_events(
+            service_client, endpoint_bridge, endpoint_url, client_config,
+            scoped_config)
         self._register_endpoint_discovery(
             service_client, endpoint_url, client_config
         )
+        self._register_lazy_block_unknown_fips_pseudo_regions(service_client)
         return service_client
 
     def create_client_class(self, service_name, api_version=None):
@@ -197,14 +204,62 @@ class ClientCreator(object):
         elif self._config_store:
             enabled = self._config_store.get_config_variable(
                 'endpoint_discovery_enabled')
-        if enabled:
-            manager = EndpointDiscoveryManager(client)
+
+        enabled = self._normalize_endpoint_discovery_config(enabled)
+        if enabled and self._requires_endpoint_discovery(client, enabled):
+            discover = enabled is True
+            manager = EndpointDiscoveryManager(client, always_discover=discover)
             handler = EndpointDiscoveryHandler(manager)
             handler.register(events, service_id)
         else:
             events.register('before-parameter-build',
                             block_endpoint_discovery_required_operations)
 
+    def _normalize_endpoint_discovery_config(self, enabled):
+        """Config must either be a boolean-string or string-literal 'auto'"""
+        if isinstance(enabled, str):
+            enabled = enabled.lower().strip()
+            if enabled == 'auto':
+                return enabled
+            elif enabled in ('true', 'false'):
+                return ensure_boolean(enabled)
+        elif isinstance(enabled, bool):
+            return enabled
+
+        raise InvalidEndpointDiscoveryConfigurationError(config_value=enabled)
+
+    def _requires_endpoint_discovery(self, client, enabled):
+        if enabled == "auto":
+            return client.meta.service_model.endpoint_discovery_required
+        return enabled
+
+    def _register_lazy_block_unknown_fips_pseudo_regions(self, client):
+        # This function blocks usage of FIPS pseudo-regions when the endpoint
+        # is not explicitly known to exist to the client to prevent accidental
+        # usage of incorrect or non-FIPS endpoints. This is done lazily by
+        # registering an exception on the before-sign event to allow for
+        # general client usage such as can_paginate, exceptions, etc.
+        region_name = client.meta.region_name
+        if not region_name or 'fips' not in region_name.lower():
+            return
+
+        partition = client.meta.partition
+        endpoint_prefix = client.meta.service_model.endpoint_prefix
+        known_regions = self._endpoint_resolver.get_available_endpoints(
+            endpoint_prefix,
+            partition,
+            allow_non_regional=True,
+        )
+
+        if region_name not in known_regions:
+            def _lazy_fips_exception(**kwargs):
+                service_name = client.meta.service_model.service_name
+                raise UnknownFIPSEndpointError(
+                    region_name=region_name,
+                    service_name=service_name,
+                )
+            client.meta.events.register('before-sign', _lazy_fips_exception)
+
     def _register_s3_events(self, client, endpoint_bridge, endpoint_url,
                             client_config, scoped_config):
         if client.meta.service_model.service_name != 's3':
@@ -221,6 +276,21 @@ class ClientCreator(object):
         self._set_s3_presign_signature_version(
             client.meta, client_config, scoped_config)
 
+    def _register_s3_control_events(
+        self, client, endpoint_bridge,
+        endpoint_url, client_config, scoped_config
+    ):
+        if client.meta.service_model.service_name != 's3control':
+            return
+        S3ControlArnParamHandler().register(client.meta.events)
+        S3ControlEndpointSetter(
+            endpoint_resolver=self._endpoint_resolver,
+            region=client.meta.region_name,
+            s3_config=client.meta.config.s3,
+            endpoint_url=endpoint_url,
+            partition=client.meta.partition
+        ).register(client.meta.events)
+
     def _set_s3_presign_signature_version(self, client_meta,
                                           client_config, scoped_config):
         # This will return the manually configured signature version, or None
@@ -359,6 +429,15 @@ class ClientEndpointBridge(object):
         region_name = self._check_default_region(service_name, region_name)
         resolved = self.endpoint_resolver.construct_endpoint(
             service_name, region_name)
+
+        # If we can't resolve the region, we'll attempt to get a global
+        # endpoint for non-regionalized services (iam, route53, etc)
+        if not resolved:
+            # TODO: fallback partition_name should be configurable in the
+            # future for users to define as needed.
+            resolved = self.endpoint_resolver.construct_endpoint(
+                service_name, region_name, partition_name='aws')
+
         if resolved:
             return self._create_endpoint(
                 resolved, service_name, region_name, endpoint_url, is_secure)
