@@ -14,7 +14,7 @@
 from __future__ import unicode_literals
 
 import requests
-from requests.packages import urllib3
+import urllib3
 
 
 def _exception_from_packed_args(exception_cls, args=None, kwargs=None):
@@ -50,7 +50,7 @@ class DataNotFoundError(BotoCoreError):
     """
     The data associated with a particular path could not be loaded.
 
-    :ivar path: The data path that the user attempted to load.
+    :ivar data_path: The data path that the user attempted to load.
     """
     fmt = 'Unable to load data for: {data_path}'
 
@@ -68,17 +68,17 @@ class UnknownServiceError(DataNotFoundError):
 
 class ApiVersionNotFoundError(BotoCoreError):
     """
-    The data associated with either that API version or a compatible one
+    The data associated with either the API version or a compatible one
     could not be loaded.
 
-    :ivar path: The data path that the user attempted to load.
-    :ivar path: The API version that the user attempted to load.
+    :ivar data_path: The data path that the user attempted to load.
+    :ivar api_version: The API version that the user attempted to load.
     """
     fmt = 'Unable to load data {data_path} for: {api_version}'
 
 
 class HTTPClientError(BotoCoreError):
-    fmt = 'An HTTP Client raised and unhandled exception: {error}'
+    fmt = 'An HTTP Client raised an unhandled exception: {error}'
     def __init__(self, request=None, response=None, **kwargs):
         self.request = request
         self.response = response
@@ -93,6 +93,15 @@ class ConnectionError(BotoCoreError):
     fmt = 'An HTTP Client failed to establish a connection: {error}'
 
 
+class InvalidIMDSEndpointError(BotoCoreError):
+    fmt = 'Invalid endpoint EC2 Instance Metadata endpoint: {endpoint}'
+
+
+class InvalidIMDSEndpointModeError(BotoCoreError):
+    fmt = ('Invalid EC2 Instance Metadata endpoint mode: {mode}'
+        ' Valid endpoint modes (case-insensitive): {valid_modes}.')
+
+
 class EndpointConnectionError(ConnectionError):
     fmt = 'Could not connect to the endpoint URL: "{endpoint_url}"'
 
@@ -122,7 +131,7 @@ class ProxyConnectionError(ConnectionError, requests.exceptions.ProxyError):
 
 class NoCredentialsError(BotoCoreError):
     """
-    No credentials could be found
+    No credentials could be found.
     """
     fmt = 'Unable to locate credentials'
 
@@ -142,7 +151,7 @@ class CredentialRetrievalError(BotoCoreError):
     Error attempting to retrieve credentials from a remote source.
 
     :ivar provider: The name of the credential provider.
-    :ivar error_msg: The msg explaning why credentials could not be
+    :ivar error_msg: The msg explaining why credentials could not be
         retrieved.
 
     """
@@ -195,6 +204,20 @@ class UnknownEndpointError(BaseEndpointResolverError, ValueError):
         '{service_name} in region {region_name}')
 
 
+class UnknownFIPSEndpointError(BaseEndpointResolverError):
+    """
+    Could not construct a FIPS endpoint.
+
+    :ivar service_name: The name of the service.
+    :ivar region_name: The name of the region.
+    """
+    fmt = (
+        'The provided FIPS pseudo-region "{region_name}" is not known for '
+        'the service "{service_name}". A FIPS compliant endpoint cannot be '
+        'constructed.'
+    )
+
+
 class ProfileNotFound(BotoCoreError):
     """
     The specified configuration profile was not found in the
@@ -262,7 +285,7 @@ class ParamValidationError(BotoCoreError):
 # error.
 class UnknownKeyError(ValidationError):
     """
-    Unknown key in a struct paramster.
+    Unknown key in a struct parameter.
 
     :ivar value: The value that was being checked.
     :ivar param: The name of the parameter.
@@ -299,6 +322,17 @@ class UnknownParameterError(ValidationError):
     )
 
 
+class InvalidRegionError(ValidationError, ValueError):
+    """
+    Invalid region_name provided to client or resource.
+
+    :ivar region_name: region_name that was being validated.
+    """
+    fmt = (
+        "Provided region_name '{region_name}' doesn't match a supported format."
+    )
+
+
 class AliasConflictParameterError(ValidationError):
     """
     Error when an alias is provided for a parameter as well as the original.
@@ -383,7 +417,7 @@ class UnknownClientMethodError(BotoCoreError):
 
 
 class UnsupportedSignatureVersionError(BotoCoreError):
-    """Error when trying to access a method on a client that does not exist."""
+    """Error when trying to use an unsupported Signature Version."""
     fmt = 'Signature version is not supported: {signature_version}'
 
 
@@ -449,19 +483,50 @@ class InvalidDNSNameError(BotoCoreError):
 class InvalidS3AddressingStyleError(BotoCoreError):
     """Error when an invalid path style is specified"""
     fmt = (
-        'S3 addressing style {s3_addressing_style} is invaild. Valid options '
+        'S3 addressing style {s3_addressing_style} is invalid. Valid options '
         'are: \'auto\', \'virtual\', and \'path\''
     )
 
 
 class UnsupportedS3ArnError(BotoCoreError):
-    """Error when S3 arn provided to Bucket parameter is not supported"""
+    """Error when S3 ARN provided to Bucket parameter is not supported"""
     fmt = (
         'S3 ARN {arn} provided to "Bucket" parameter is invalid. Only '
         'ARNs for S3 access-points are supported.'
     )
 
 
+class UnsupportedS3ControlArnError(BotoCoreError):
+    """Error when S3 ARN provided to S3 control parameter is not supported"""
+    fmt = (
+        'S3 ARN "{arn}" provided is invalid for this operation. {msg}'
+    )
+
+
+class InvalidHostLabelError(BotoCoreError):
+    """Error when an invalid host label would be bound to an endpoint"""
+    fmt = (
+        'Invalid host label to be bound to the hostname of the endpoint: '
+        '"{label}".'
+    )
+
+
+class UnsupportedOutpostResourceError(BotoCoreError):
+    """Error when S3 Outpost ARN provided to Bucket parameter is incomplete"""
+    fmt = (
+        'S3 Outpost ARN resource "{resource_name}" provided to "Bucket" '
+        'parameter is invalid. Only ARNs for S3 Outpost arns with an '
+        'access-point sub-resource are supported.'
+    )
+
+
+class UnsupportedS3ConfigurationError(BotoCoreError):
+    """Error when an unsupported configuration is used with access-points"""
+    fmt = (
+        'Unsupported configuration when using S3: {msg}'
+    )
+
+
 class UnsupportedS3AccesspointConfigurationError(BotoCoreError):
     """Error when an unsupported configuration is used with access-points"""
     fmt = (
@@ -469,6 +534,21 @@ class UnsupportedS3AccesspointConfigurationError(BotoCoreError):
     )
 
 
+class InvalidEndpointDiscoveryConfigurationError(BotoCoreError):
+    """Error when invalid value supplied for endpoint_discovery_enabled"""
+    fmt = (
+        'Unsupported configuration value for endpoint_discovery_enabled. '
+        'Expected one of ("true", "false", "auto") but got {config_value}.'
+    )
+
+
+class UnsupportedS3ControlConfigurationError(BotoCoreError):
+    """Error when an unsupported configuration is used with S3 Control"""
+    fmt = (
+        'Unsupported configuration when using S3 Control: {msg}'
+    )
+
+
 class InvalidRetryConfigurationError(BotoCoreError):
     """Error when invalid retry configuration is specified"""
     fmt = (
@@ -498,7 +578,7 @@ class InvalidS3UsEast1RegionalEndpointConfigError(BotoCoreError):
     fmt = (
         'S3 us-east-1 regional endpoint option '
         '{s3_us_east_1_regional_endpoint_config} is '
-        'invaild. Valid options are: legacy and regional'
+        'invalid. Valid options are: "legacy", "regional"'
     )
 
 
@@ -506,7 +586,7 @@ class InvalidSTSRegionalEndpointsConfigError(BotoCoreError):
     """Error when invalid sts regional endpoints configuration is specified"""
     fmt = (
         'STS regional endpoints option {sts_regional_endpoints_config} is '
-        'invaild. Valid options are: legacy and regional'
+        'invalid. Valid options are: "legacy", "regional"'
     )
 
 
@@ -517,9 +597,11 @@ class StubResponseError(BotoCoreError):
 class StubAssertionError(StubResponseError, AssertionError):
     pass
 
+
 class UnStubbedResponseError(StubResponseError):
     pass
 
+
 class InvalidConfigError(BotoCoreError):
     fmt = '{error_msg}'
 
@@ -540,6 +622,10 @@ class MD5UnavailableError(BotoCoreError):
     fmt = "This system does not support MD5 generation."
 
 
+class MissingDependencyException(BotoCoreError):
+    fmt = "Missing Dependency: {msg}"
+
+
 class MetadataRetrievalError(BotoCoreError):
     fmt = "Error retrieving metadata: {error_msg}"
 
@@ -560,7 +646,29 @@ class MissingServiceIdError(UndefinedModelAttributeError):
         self.kwargs = kwargs
 
 
+class SSOError(BotoCoreError):
+    fmt = "An unspecified error happened when resolving SSO credentials"
+
+
+class SSOTokenLoadError(SSOError):
+    fmt = "Error loading SSO Token: {error_msg}"
+
+
+class UnauthorizedSSOTokenError(SSOError):
+    fmt = (
+        "The SSO session associated with this profile has expired or is "
+        "otherwise invalid. To refresh this SSO session run aws sso login "
+        "with the corresponding profile."
+    )
+
+
 class CapacityNotAvailableError(BotoCoreError):
     fmt = (
         'Insufficient request capacity available.'
     )
+
+
+class InvalidProxiesConfigError(BotoCoreError):
+    fmt = (
+        'Invalid configuration value(s) provided for proxies_config.'
+    )
