@@ -117,10 +117,9 @@ Each call to ``parse()`` returns a dict has this form::
 import re
 import base64
 import json
-import xml.etree.cElementTree
 import logging
 
-from ibm_botocore.compat import six, XMLParseError
+from ibm_botocore.compat import six, ETree, XMLParseError
 from ibm_botocore.eventstream import EventStream, NoInitialResponseError
 
 from ibm_botocore.utils import parse_timestamp, merge_dicts, \
@@ -141,7 +140,7 @@ class ResponseParserFactory(object):
         You can specify any kwargs that are allowed by a ResponseParser
         class.  There are currently two arguments:
 
-            * timestamp_parser - A callable that can parse a timetsamp string
+            * timestamp_parser - A callable that can parse a timestamp string
             * blob_parser - A callable that can parse a blob type
 
         """
@@ -236,6 +235,10 @@ class ResponseParser(object):
         if response['status_code'] >= 301:
             if self._is_generic_error_response(response):
                 parsed = self._do_generic_error_parse(response)
+            elif self._is_modeled_error_shape(shape):
+                parsed = self._do_modeled_error_parse(response, shape)
+                # We don't want to decorate the modeled fields with metadata
+                return parsed
             else:
                 parsed = self._do_error_parse(response, shape)
         else:
@@ -258,6 +261,9 @@ class ResponseParser(object):
             parsed['ResponseMetadata'] = response_metadata
         return parsed
 
+    def _is_modeled_error_shape(self, shape):
+        return shape is not None and shape.metadata.get('exception', False)
+
     def _is_generic_error_response(self, response):
         # There are times when a service will respond with a generic
         # error response such as:
@@ -296,6 +302,10 @@ class ResponseParser(object):
         raise NotImplementedError(
             "%s._do_error_parse" % self.__class__.__name__)
 
+    def _do_modeled_error_parse(self, response, shape, parsed):
+        raise NotImplementedError(
+            "%s._do_modeled_error_parse" % self.__class__.__name__)
+
     def _parse_shape(self, shape, node):
         handler = getattr(self, '_handle_%s' % shape.type_name,
                           self._default_handle)
@@ -318,6 +328,31 @@ class ResponseParser(object):
         name = response['context'].get('operation_name')
         return EventStream(response['body'], shape, parser, name)
 
+    def _get_first_key(self, value):
+        return list(value)[0]
+
+    def _has_unknown_tagged_union_member(self, shape, value):
+        if shape.is_tagged_union:
+            if len(value) != 1:
+                error_msg = (
+                    "Invalid service response: %s must only have one member "
+                    "set."
+                )
+                raise ResponseParserError(error_msg % shape.name)
+            tag = self._get_first_key(value)
+            if tag not in shape.members:
+                msg = (
+                    "Received a tagged union response with member "
+                    "unknown to client: %s. Please upgrade SDK for full "
+                    "response support."
+                )
+                LOG.info(msg % tag)
+                return True
+        return False
+
+    def _handle_unknown_tagged_union_member(self, tag):
+        return {'SDK_UNKNOWN_MEMBER': {'name': tag}}
+
 
 class BaseXMLResponseParser(ResponseParser):
     def __init__(self, timestamp_parser=None, blob_parser=None):
@@ -362,7 +397,12 @@ class BaseXMLResponseParser(ResponseParser):
     def _handle_structure(self, shape, node):
         parsed = {}
         members = shape.members
+        if shape.metadata.get('exception', False):
+            node = self._get_error_root(node)
         xml_dict = self._build_name_to_xml_node(node)
+        if self._has_unknown_tagged_union_member(shape, xml_dict):
+            tag = self._get_first_key(xml_dict)
+            return self._handle_unknown_tagged_union_member(tag)
         for member_name in members:
             member_shape = members[member_name]
             if 'location' in member_shape.serialization or \
@@ -386,6 +426,13 @@ class BaseXMLResponseParser(ResponseParser):
                     parsed[member_name] = attribs[location_name]
         return parsed
 
+    def _get_error_root(self, original_root):
+        if self._node_tag(original_root) == 'ErrorResponse':
+            for child in original_root:
+                if self._node_tag(child) == 'Error':
+                    return child
+        return original_root
+
     def _member_key_name(self, shape, member_name):
         # This method is needed because we have to special case flattened list
         # with a serialization name.  If this is the case we use the
@@ -426,15 +473,16 @@ class BaseXMLResponseParser(ResponseParser):
 
     def _parse_xml_string_to_dom(self, xml_string):
         try:
-            parser = xml.etree.cElementTree.XMLParser(
-                target=xml.etree.cElementTree.TreeBuilder(),
+            parser = ETree.XMLParser(
+                target=ETree.TreeBuilder(),
                 encoding=self.DEFAULT_ENCODING)
             parser.feed(xml_string)
             root = parser.close()
         except XMLParseError as e:
             raise ResponseParserError(
                 "Unable to parse response (%s), "
-                "invalid XML received:\n%s" % (e, xml_string))
+                "invalid XML received. Further retries may succeed:\n%s" %
+                (e, xml_string))
         return root
 
     def _replace_nodes(self, parsed):
@@ -496,7 +544,13 @@ class QueryParser(BaseXMLResponseParser):
             parsed['ResponseMetadata'] = {'RequestId': parsed.pop('RequestId')}
         return parsed
 
+    def _do_modeled_error_parse(self, response, shape):
+        return self._parse_body_as_xml(response, shape, inject_metadata=False)
+
     def _do_parse(self, response, shape):
+        return self._parse_body_as_xml(response, shape, inject_metadata=True)
+
+    def _parse_body_as_xml(self, response, shape, inject_metadata=True):
         xml_contents = response['body']
         root = self._parse_xml_string_to_dom(xml_contents)
         parsed = {}
@@ -507,7 +561,8 @@ class QueryParser(BaseXMLResponseParser):
                     shape.serialization['resultWrapper'],
                     root)
             parsed = self._parse_shape(shape, start)
-        self._inject_response_metadata(root, parsed)
+        if inject_metadata:
+            self._inject_response_metadata(root, parsed)
         return parsed
 
     def _find_result_wrapped_shape(self, element_name, xml_root_node):
@@ -546,30 +601,46 @@ class EC2QueryParser(QueryParser):
         # This is different from QueryParser in that it's RequestID,
         # not RequestId
         original = super(EC2QueryParser, self)._do_error_parse(response, shape)
-        original['ResponseMetadata'] = {
-            'RequestId': original.pop('RequestID')
-        }
+        if 'RequestID' in original:
+            original['ResponseMetadata'] = {
+                'RequestId': original.pop('RequestID')
+            }
         return original
 
+    def _get_error_root(self, original_root):
+        for child in original_root:
+            if self._node_tag(child) == 'Errors':
+                for errors_child in child:
+                    if self._node_tag(errors_child) == 'Error':
+                        return errors_child
+        return original_root
+
 
 class BaseJSONParser(ResponseParser):
 
     def _handle_structure(self, shape, value):
-        member_shapes = shape.members
-        if value is None:
-            # If the comes across the wire as "null" (None in python),
-            # we should be returning this unchanged, instead of as an
-            # empty dict.
-            return None
         final_parsed = {}
-        for member_name in member_shapes:
-            member_shape = member_shapes[member_name]
-            json_name = member_shape.serialization.get('name', member_name)
-            raw_value = value.get(json_name)
-            if raw_value is not None:
-                final_parsed[member_name] = self._parse_shape(
-                    member_shapes[member_name],
-                    raw_value)
+        if shape.is_document_type:
+            final_parsed = value
+        else:
+            member_shapes = shape.members
+            if value is None:
+                # If the comes across the wire as "null" (None in python),
+                # we should be returning this unchanged, instead of as an
+                # empty dict.
+                return None
+            final_parsed = {}
+            if self._has_unknown_tagged_union_member(shape, value):
+                tag = self._get_first_key(value)
+                return self._handle_unknown_tagged_union_member(tag)
+            for member_name in member_shapes:
+                member_shape = member_shapes[member_name]
+                json_name = member_shape.serialization.get('name', member_name)
+                raw_value = value.get(json_name)
+                if raw_value is not None:
+                    final_parsed[member_name] = self._parse_shape(
+                        member_shapes[member_name],
+                        raw_value)
         return final_parsed
 
     def _handle_map(self, shape, value):
@@ -721,7 +792,7 @@ class EventStreamXMLParser(BaseEventStreamParser, BaseXMLResponseParser):
 
     def _initial_body_parse(self, xml_string):
         if not xml_string:
-            return xml.etree.cElementTree.Element('')
+            return ETree.Element('')
         return self._parse_xml_string_to_dom(xml_string)
 
 
@@ -741,6 +812,9 @@ class JSONParser(BaseJSONParser):
         self._inject_response_metadata(parsed, response['headers'])
         return parsed
 
+    def _do_modeled_error_parse(self, response, shape):
+        return self._handle_json_body(response['body'], shape)
+
     def _handle_event_stream(self, response, shape, event_name):
         event_stream_shape = shape.members[event_name]
         event_stream = self._create_event_stream(response, event_stream_shape)
@@ -767,12 +841,20 @@ class BaseRestParser(ResponseParser):
         final_parsed = {}
         final_parsed['ResponseMetadata'] = self._populate_response_metadata(
             response)
+        self._add_modeled_parse(response, shape, final_parsed)
+        return final_parsed
+
+    def _add_modeled_parse(self, response, shape, final_parsed):
         if shape is None:
             return final_parsed
         member_shapes = shape.members
         self._parse_non_payload_attrs(response, shape,
                                       member_shapes, final_parsed)
         self._parse_payload(response, shape, member_shapes, final_parsed)
+
+    def _do_modeled_error_parse(self, response, shape):
+        final_parsed = {}
+        self._add_modeled_parse(response, shape, final_parsed)
         return final_parsed
 
     def _populate_response_metadata(self, response):
@@ -893,7 +975,7 @@ class RestXMLParser(BaseRestParser, BaseXMLResponseParser):
 
     def _initial_body_parse(self, xml_string):
         if not xml_string:
-            return xml.etree.cElementTree.Element('')
+            return ETree.Element('')
         return self._parse_xml_string_to_dom(xml_string)
 
     def _do_error_parse(self, response, shape):
