@@ -49,6 +49,7 @@ from ibm_botocore.compat import json, formatdate
 from ibm_botocore.utils import parse_to_aware_datetime
 from ibm_botocore.utils import percent_encode
 from ibm_botocore.utils import is_json_value_header
+from ibm_botocore.utils import conditionally_calculate_md5
 from ibm_botocore import validate
 
 
@@ -184,6 +185,12 @@ class Serializer(object):
 
         return host_prefix_expression.format(**format_kwargs)
 
+    def _prepare_additional_traits(self, request, operation_model):
+        """Determine if additional traits are required for given model"""
+        if operation_model.http_checksum_required:
+            conditionally_calculate_md5(request)
+        return request
+
 
 class QuerySerializer(Serializer):
 
@@ -210,6 +217,8 @@ class QuerySerializer(Serializer):
         if host_prefix is not None:
             serialized['host_prefix'] = host_prefix
 
+        serialized = self._prepare_additional_traits(serialized,
+                operation_model)
         return serialized
 
     def _serialize(self, serialized, value, shape, prefix=''):
@@ -343,6 +352,8 @@ class JSONSerializer(Serializer):
         if host_prefix is not None:
             serialized['host_prefix'] = host_prefix
 
+        serialized = self._prepare_additional_traits(serialized,
+                operation_model)
         return serialized
 
     def _serialize(self, serialized, value, shape, key=None):
@@ -351,21 +362,24 @@ class JSONSerializer(Serializer):
         method(serialized, value, shape, key)
 
     def _serialize_type_structure(self, serialized, value, shape, key):
-        if key is not None:
-            # If a key is provided, this is a result of a recursive
-            # call so we need to add a new child dict as the value
-            # of the passed in serialized dict.  We'll then add
-            # all the structure members as key/vals in the new serialized
-            # dictionary we just created.
-            new_serialized = self.MAP_TYPE()
-            serialized[key] = new_serialized
-            serialized = new_serialized
-        members = shape.members
-        for member_key, member_value in value.items():
-            member_shape = members[member_key]
-            if 'name' in member_shape.serialization:
-                member_key = member_shape.serialization['name']
-            self._serialize(serialized, member_value, member_shape, member_key)
+        if shape.is_document_type:
+            serialized[key] = value
+        else:
+            if key is not None:
+                # If a key is provided, this is a result of a recursive
+                # call so we need to add a new child dict as the value
+                # of the passed in serialized dict.  We'll then add
+                # all the structure members as key/vals in the new serialized
+                # dictionary we just created.
+                new_serialized = self.MAP_TYPE()
+                serialized[key] = new_serialized
+                serialized = new_serialized
+            members = shape.members
+            for member_key, member_value in value.items():
+                member_shape = members[member_key]
+                if 'name' in member_shape.serialization:
+                    member_key = member_shape.serialization['name']
+                self._serialize(serialized, member_value, member_shape, member_key)
 
     def _serialize_type_map(self, serialized, value, shape, key):
         map_obj = self.MAP_TYPE()
@@ -460,6 +474,8 @@ class BaseRestSerializer(Serializer):
         if host_prefix is not None:
             serialized['host_prefix'] = host_prefix
 
+        serialized = self._prepare_additional_traits(serialized,
+                operation_model)
         return serialized
 
     def _render_uri_template(self, uri_template, params):
