@@ -34,9 +34,9 @@ from nose.tools import assert_equal
 import ibm_botocore.loaders
 import ibm_botocore.session
 from ibm_botocore.awsrequest import AWSResponse
-from ibm_botocore.compat import six
-from ibm_botocore.compat import urlparse
-from ibm_botocore.compat import parse_qs
+from ibm_botocore.compat import (
+    parse_qs, six, urlparse, HAS_CRT
+)
 from ibm_botocore import utils
 from ibm_botocore import credentials
 from ibm_botocore.stub import Stubber
@@ -71,6 +71,15 @@ def skip_if_windows(reason):
     return decorator
 
 
+def requires_crt(reason=None):
+    if reason is None:
+        reason = "Test requires awscrt to be installed"
+
+    def decorator(func):
+        return unittest.skipIf(not HAS_CRT, reason)(func)
+    return decorator
+
+
 def random_chars(num_chars):
     """Returns random hex characters.
 
@@ -514,9 +523,9 @@ class StubbedSession(ibm_botocore.session.Session):
         self._client_stubs[service_name] = stubber
         return client
 
-    def stub(self, service_name):
+    def stub(self, service_name, *args, **kwargs):
         if service_name not in self._client_stubs:
-            self.create_client(service_name)
+            self.create_client(service_name, *args, **kwargs)
         return self._client_stubs[service_name]
 
     def activate_stubs(self):
