@@ -28,11 +28,11 @@ from ibm_botocore.awsrequest import AWSRequest
 
 class BaseTestWithFixedDate(unittest.TestCase):
     def setUp(self):
-        self.datetime_patch = mock.patch('ibm_botocore.auth.datetime')
-        self.datetime_mock = self.datetime_patch.start()
         self.fixed_date = datetime.datetime(2014, 3, 10, 17, 2, 55, 0)
-        self.datetime_mock.datetime.utcnow.return_value = self.fixed_date
-        self.datetime_mock.datetime.strptime.return_value = self.fixed_date
+        self.datetime_patch = mock.patch('ibm_botocore.auth.datetime.datetime')
+        self.datetime_mock = self.datetime_patch.start()
+        self.datetime_mock.utcnow.return_value = self.fixed_date
+        self.datetime_mock.strptime.return_value = self.fixed_date
 
     def tearDown(self):
         self.datetime_patch.stop()
@@ -285,13 +285,14 @@ class TestSigV3(unittest.TestCase):
 
 class TestS3SigV4Auth(BaseTestWithFixedDate):
 
+    AuthClass = ibm_botocore.auth.S3SigV4Auth
     maxDiff = None
 
     def setUp(self):
         super(TestS3SigV4Auth, self).setUp()
         self.credentials = ibm_botocore.credentials.Credentials(
             access_key='foo', secret_key='bar', token='baz')
-        self.auth = ibm_botocore.auth.S3SigV4Auth(
+        self.auth = self.AuthClass(
             self.credentials, 'ec2', 'eu-central-1')
         self.request = AWSRequest(data=six.BytesIO(b"foo bar baz"))
         self.request.method = 'PUT'
@@ -319,12 +320,16 @@ class TestS3SigV4Auth(BaseTestWithFixedDate):
         request.method = 'GET'
         credentials = ibm_botocore.credentials.Credentials('access_key',
                                                        'secret_key')
-        auth = ibm_botocore.auth.S3SigV4Auth(credentials, 's3', 'us-east-1')
+        auth = self.AuthClass(credentials, 's3', 'us-east-1')
         auth.add_auth(request)
         self.assertTrue(
             request.headers['Authorization'].startswith('AWS4-HMAC-SHA256'))
 
     def test_query_string_params_in_urls(self):
+        if not hasattr(self.AuthClass, 'canonical_query_string'):
+            raise unittest.SkipTest('%s does not expose interim steps' %
+                                    self.AuthClass.__name__)
+
         request = AWSRequest()
         request.url = (
             'https://s3.amazonaws.com/bucket?'
@@ -346,7 +351,7 @@ class TestS3SigV4Auth(BaseTestWithFixedDate):
         request.headers[header] = value
         credentials = ibm_botocore.credentials.Credentials('access_key',
                                                        'secret_key')
-        auth = ibm_botocore.auth.S3SigV4Auth(credentials, 's3', 'us-east-1')
+        auth = self.AuthClass(credentials, 's3', 'us-east-1')
         auth.add_auth(request)
         self.assertNotIn(header, request.headers['Authorization'])
 
@@ -584,13 +589,13 @@ class TestSigV4(unittest.TestCase):
 class TestSigV4Resign(BaseTestWithFixedDate):
 
     maxDiff = None
+    AuthClass = ibm_botocore.auth.SigV4Auth
 
     def setUp(self):
         super(TestSigV4Resign, self).setUp()
         self.credentials = ibm_botocore.credentials.Credentials(
             access_key='foo', secret_key='bar', token='baz')
-        self.auth = ibm_botocore.auth.SigV4Auth(self.credentials,
-                                            'ec2', 'us-west-2')
+        self.auth = self.AuthClass(self.credentials, 'ec2', 'us-west-2')
         self.request = AWSRequest()
         self.request.method = 'PUT'
         self.request.url = 'https://ec2.amazonaws.com/'
@@ -708,6 +713,7 @@ class TestS3SigV2Presign(BasePresignTest):
 class TestSigV4Presign(BasePresignTest):
 
     maxDiff = None
+    AuthClass = ibm_botocore.auth.SigV4QueryAuth
 
     def setUp(self):
         self.access_key = 'access_key'
@@ -716,7 +722,7 @@ class TestSigV4Presign(BasePresignTest):
                                                             self.secret_key)
         self.service_name = 'myservice'
         self.region_name = 'myregion'
-        self.auth = ibm_botocore.auth.SigV4QueryAuth(
+        self.auth = self.AuthClass(
             self.credentials, self.service_name, self.region_name, expires=60)
         self.datetime_patcher = mock.patch.object(
             ibm_botocore.auth.datetime, 'datetime',
