@@ -377,6 +377,130 @@ class TestResponseMetadataParsed(unittest.TestCase):
         self.assertEqual(parsed, expected)
 
 
+class TestTaggedUnions(unittest.TestCase):
+    def assert_tagged_union_response_with_unknown_member(self,
+                                                         parser,
+                                                         response,
+                                                         output_shape,
+                                                         expected_parsed_response,
+                                                         expected_log):
+        with self.assertLogs() as captured_log:
+            parsed = parser.parse(response, output_shape)
+            self.assertEqual(parsed, expected_parsed_response)
+            self.assertEqual(len(captured_log.records), 1)
+            self.assertIn(('Received a tagged union response with member '
+                           'unknown to client'),
+                          captured_log.records[0].getMessage())
+
+    def test_base_json_parser_handles_unknwon_member(self):
+        parser = parsers.JSONParser()
+        response = b'{"Foo": "mystring"}'
+        headers = {'x-amzn-requestid': 'request-id'}
+        output_shape = model.StructureShape(
+            'OutputShape',
+            {
+                'type': 'structure',
+                'union':True,
+                'members': {
+                    'Str': {
+                        'shape': 'StringType',
+                    }
+                }
+            },
+            model.ShapeResolver({'StringType': {'type': 'string'}})
+        )
+        response = {'body': response, 'headers': headers, 'status_code': 200}
+        # Parsed response omits data from service since it is not
+        # modeled in the client
+        expected_parsed_response = {
+            'SDK_UNKNOWN_MEMBER': {
+                'name': 'Foo'
+            },
+            'ResponseMetadata':
+            {'RequestId': 'request-id',
+             'HTTPStatusCode': 200,
+             'HTTPHeaders': {
+                 'x-amzn-requestid': 'request-id'}
+             }
+        }
+        expected_log = "Received a response with an unknown member Foo set"
+        self.assert_tagged_union_response_with_unknown_member(
+            parser, response, output_shape, expected_parsed_response,
+            expected_log
+        )
+
+    def test_base_xml_parser_handles_unknown_member(self):
+        parser = parsers.QueryParser()
+        response = (
+            '<OperationNameResponse>'
+            '  <OperationNameResult><Foo>mystring</Foo></OperationNameResult>'
+            '  <ResponseMetadata>'
+            '    <RequestId>request-id</RequestId>'
+            '  </ResponseMetadata>'
+            '</OperationNameResponse>').encode('utf-8')
+        output_shape = model.StructureShape(
+            'OutputShape',
+            {
+                'type': 'structure',
+                'union':True,
+                'resultWrapper': 'OperationNameResult',
+                'members': {
+                    'Str': {
+                        'shape': 'StringType',
+                    },
+                }
+            },
+            model.ShapeResolver({
+                'StringType': {
+                    'type': 'string',
+                },
+            })
+        )
+        response = {'body': response, 'headers': {}, 'status_code': 200}
+        # Parsed response omits data from service since it is not
+        # modeled in the client
+        expected_parsed_response = {
+            'SDK_UNKNOWN_MEMBER': {
+                'name': 'Foo'
+            },
+            'ResponseMetadata':{
+                'RequestId': 'request-id',
+                'HTTPStatusCode': 200,
+                'HTTPHeaders': {}
+            }
+        }
+        expected_log = "Received a response with an unknown member Foo set"
+        self.assert_tagged_union_response_with_unknown_member(
+            parser, response, output_shape, expected_parsed_response,
+            expected_log
+        )
+
+    def test_parser_errors_out_when_multiple_members_set(self):
+        parser = parsers.JSONParser()
+        response = b'{"Foo": "mystring", "Bar": "mystring2"}'
+        headers = {'x-amzn-requestid': 'request-id'}
+        output_shape = model.StructureShape(
+            'OutputShape',
+            {
+                'type': 'structure',
+                'union':True,
+                'members': {
+                    'Foo': {
+                        'shape': 'StringType',
+                    },
+                    'Bar': {
+                        'shape': 'StringType',
+                    }
+                }
+            },
+            model.ShapeResolver({'StringType': {'type': 'string'}})
+        )
+
+        response = {'body': response, 'headers': headers, 'status_code': 200}
+        with self.assertRaises(parsers.ResponseParserError):
+            parser.parse(response, output_shape)
+
+
 class TestHeaderResponseInclusion(unittest.TestCase):
     def create_parser(self):
         return parsers.JSONParser()
@@ -597,8 +721,8 @@ class TestHandlesInvalidXMLResponses(unittest.TestCase):
         parser = parsers.QueryParser()
         output_shape = None
         # The XML body should be in the error message.
-        with self.assertRaisesRegexp(parsers.ResponseParserError,
-                                     '<DeleteTagsResponse'):
+        with self.assertRaisesRegex(parsers.ResponseParserError,
+                                    '<DeleteTagsResponse'):
             parser.parse(
                 {'body': invalid_xml, 'headers': {}, 'status_code': 200},
                 output_shape)
@@ -862,6 +986,21 @@ class TestParseErrorResponses(unittest.TestCase):
     # this into the shared protocol tests in the future,
     # so consolidating them into a single class will make
     # this easier.
+    def setUp(self):
+        self.error_shape = model.StructureShape(
+            'ErrorShape',
+            {
+                'type': 'structure',
+                'exception': True,
+                'members': {
+                    'ModeledField': {
+                        'shape': 'StringType',
+                    }
+                }
+            },
+            model.ShapeResolver({'StringType': {'type': 'string'}})
+        )
+
     def test_response_metadata_errors_for_json_protocol(self):
         parser = parsers.JSONParser()
         response = {
@@ -1165,6 +1304,103 @@ class TestParseErrorResponses(unittest.TestCase):
         self.assertEqual(parsed['Error'], {'Message': 'Access denied',
                                            'Code': 'AccessDeniedException'})
 
+    def test_can_parse_rest_json_modeled_fields(self):
+        body = (
+            b'{"ModeledField":"Some modeled field",'
+            b'"Message":"Some message"}'
+        )
+        parser = parsers.RestJSONParser()
+        response_dict = {
+            'status_code': 400,
+            'headers': {},
+            'body': body,
+        }
+        parsed = parser.parse(response_dict, self.error_shape)
+        expected_parsed = {
+            'ModeledField': 'Some modeled field',
+        }
+        self.assertEqual(parsed, expected_parsed)
+
+    def test_can_parse_rest_xml_modeled_fields(self):
+        parser = parsers.RestXMLParser()
+        body = (
+            b'<?xml version="1.0"?>\n<ErrorResponse xmlns="http://foo.bar">'
+            b'<Error><Type>Sender</Type><Code>NoSuchDistribution</Code>'
+            b'<Message>The specified distribution does not exist.</Message>'
+            b'<ModeledField>Some modeled field</ModeledField>'
+            b'</Error>'
+            b'</ErrorResponse>'
+        )
+        response_dict = {
+            'status_code': 400,
+            'headers': {},
+            'body': body,
+        }
+        parsed = parser.parse(response_dict, self.error_shape)
+        expected_parsed = {
+            'ModeledField': 'Some modeled field',
+        }
+        self.assertEqual(parsed, expected_parsed)
+
+    def test_can_parse_ec2_modeled_fields(self):
+        body = (
+            b'<Response><Errors><Error>'
+            b'<Code>ExceptionShape</Code>'
+            b'<Message>Foo message</Message>'
+            b'<ModeledField>Some modeled field</ModeledField>'
+            b'</Error></Errors></Response>'
+        )
+        parser = parsers.EC2QueryParser()
+        response_dict = {
+            'status_code': 400,
+            'headers': {},
+            'body': body,
+        }
+        parsed = parser.parse(response_dict, self.error_shape)
+        expected_parsed = {
+            'ModeledField': 'Some modeled field',
+        }
+        self.assertEqual(parsed, expected_parsed)
+
+    def test_can_parse_query_modeled_fields(self):
+        parser = parsers.QueryParser()
+        body = (
+            b'<?xml version="1.0"?>\n<ErrorResponse xmlns="http://foo.bar">'
+            b'<Error><Type>Sender</Type><Code>SomeCode</Code>'
+            b'<Message>A message</Message>'
+            b'<ModeledField>Some modeled field</ModeledField>'
+            b'</Error>'
+            b'</ErrorResponse>'
+        )
+        response_dict = {
+            'status_code': 400,
+            'headers': {},
+            'body': body,
+        }
+        parsed = parser.parse(response_dict, self.error_shape)
+        expected_parsed = {
+            'ModeledField': 'Some modeled field',
+        }
+        self.assertEqual(parsed, expected_parsed)
+
+    def test_can_parse_json_modeled_fields(self):
+        body = (
+            b'{"ModeledField":"Some modeled field",'
+            b'"Message":"Some message",'
+            b'"__type": "Prefix#SomeError"}'
+        )
+        parser = parsers.JSONParser()
+        response_dict = {
+            'status_code': 400,
+            'headers': {},
+            'body': body,
+        }
+        parsed = parser.parse(response_dict, self.error_shape)
+        expected_parsed = {
+            'ModeledField': 'Some modeled field',
+        }
+        self.assertEqual(parsed, expected_parsed)
+
     def test_can_parse_route53_with_missing_message(self):
         # The message isn't always in the XML response (or even the headers).
         # We should be able to handle this gracefully and still at least
